/* settings-utils.h
 *
 * Copyright (C) 2002 Rodrigo Moya
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

#include <gtk/gtktogglebutton.h>
#include <libgnomedb/gnome-db-config.h>
#include <libgnomedb/gnome-db-util.h>
#include "settings-utils.h"

static void
check_button_conf_changed (GConfClient *conf_client,
			   guint lid,
			   GConfEntry *entry,
			   gpointer user_data)
{
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (user_data),
				      gconf_value_get_bool (gconf_entry_get_value (entry)));
}

static void
check_button_destroyed_cb (GtkWidget *w, gpointer user_data)
{
	gnome_db_config_remove_listener (GPOINTER_TO_INT (user_data));
}

static void
check_button_toggled (GtkToggleButton *button, gpointer user_data)
{
	gnome_db_config_set_boolean ((const gchar *) user_data, gtk_toggle_button_get_active (button));
}

GtkWidget *
settings_utils_new_check_button (const gchar *label, const gchar *key)
{
	GtkWidget *w;
	guint lid;

	w = gnome_db_new_check_button_widget (label);
	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
				      gnome_db_config_get_boolean (key));
	g_signal_connect (G_OBJECT (w), "toggled",
			  G_CALLBACK (check_button_toggled), key);

	lid = gnome_db_config_add_listener (key, (GConfClientNotifyFunc) check_button_conf_changed, w);
	g_signal_connect (G_OBJECT (w), "destroy",
			  G_CALLBACK (check_button_destroyed_cb), GINT_TO_POINTER (lid));

	return w;
}
